"""The exceptions for the messaging API."""


class SmsBaseException(Exception):
    """The base class for all other exceptions."""


class CredentialError(SmsBaseException):
    """Raised when required credentials are not provided."""


class SubscriptionError(SmsBaseException):
    """Raised when a subscription request failed."""


class SmsError(SmsBaseException):
    """Raised when a sms request failed."""


class BnumError(SmsBaseException):
    """Raised when a bnum request failed."""



"""Configuration for requests."""

import os
import typing

from .. import exceptions

TTlsClientKey = str
TTlsClientSecret = str


class Config:
    """
    Configuration for requests.

    Attrs:
        tls_client_key: The OAuth client id
        tls_client_secret: The OAuth client secret

    """

    def __init__(
        self,
        *,
        tls_client_key: typing.Optional[TTlsClientKey] = None,
        tls_client_secret: typing.Optional[TTlsClientSecret] = None,
    ) -> None:
        """Construct."""
        if tls_client_key is not None:
            self.tls_client_key = tls_client_key
        if tls_client_secret is not None:
            self.tls_client_secret = tls_client_secret

    _tls_client_key: typing.Optional[TTlsClientKey] = None
    _tls_client_secret: typing.Optional[TTlsClientSecret] = None

    @property
    def tls_client_key(self) -> TTlsClientKey:
        """Get the tls_client_key."""
        if self._tls_client_key is None:
            tls_client_key_env_name = "TLS_CLIENT_KEY"
            tls_client_key = os.getenv(tls_client_key_env_name)
            if not isinstance(tls_client_key, TTlsClientKey):
                raise exceptions.CredentialError(
                    "The client key was not configured. "
                    "It can be retrieved from here: "
                    "https://dev.telstra.com/user/me/apps. "
                    f"Then it can be provided as the '{tls_client_key_env_name}' "
                    "environment variable or "
                    "using the following code: \n"
                    "from tls.messaging.utils import CONFIG\n"
                    "CONFIG.tls_client_key = '<client key>'"
                )
            self._tls_client_key = tls_client_key
        return self._tls_client_key

    @tls_client_key.setter
    def tls_client_key(self, tls_client_key: TTlsClientKey) -> None:
        """Set the tls_client_key."""
        if not isinstance(tls_client_key, TTlsClientKey):
            raise exceptions.CredentialError(
                "The provided client key is not valid, "
                f"expecting a {TTlsClientKey}, "
                f"received {tls_client_key}."
            )
        self._tls_client_key = tls_client_key

    @property
    def tls_client_secret(self) -> TTlsClientSecret:
        """Get the tls_client_secret."""
        if self._tls_client_secret is None:
            tls_client_secret_env_name = "TLS_CLIENT_SECRET"
            tls_client_secret = os.getenv(tls_client_secret_env_name)
            if not isinstance(tls_client_secret, TTlsClientSecret):
                raise exceptions.CredentialError(
                    "The client secret was not configured. "
                    "It can be retrieved from here: "
                    "https://dev.telstra.com/user/me/apps. "
                    f"Then it can be provided as the '{tls_client_secret_env_name}'' "
                    "environment variable or "
                    "using the following code: \n"
                    "from tls.messaging.utils import CONFIG\n"
                    "CONFIG.tls_client_secret = '<client secret>'"
                )
            self._tls_client_secret = tls_client_secret
        return self._tls_client_secret

    @tls_client_secret.setter
    def tls_client_secret(self, tls_client_secret: TTlsClientSecret) -> None:
        """Set the tls_client_secret."""
        if not isinstance(tls_client_secret, TTlsClientSecret):
            raise exceptions.CredentialError(
                "The provided client secret is not valid, "
                f"expecting a {TTlsClientSecret}, "
                f"received {tls_client_secret}."
            )
        self._tls_client_secret = tls_client_secret


CONFIG = Config()


def get() -> Config:
    """Get the configuration."""
    return CONFIG



"""Used to send messages."""

import dataclasses
import json
import re
import typing
from urllib import error, request

from tls.messaging.utils import phone_number

from . import exceptions, oauth
from .utils import phone_number


@dataclasses.dataclass
class TSms:
    """
    A sms.

    Attrs:
        to: The destination.
        delivery_status: Whether the delivery has been completed.
        message_id: Unique identifier.
        message_status_url: URL to retrieve the current delivery status.

    """

    to: typing.Union[str, typing.List[str]]
    delivery_status: str
    message_id: str
    message_status_url: str


_VALID_FROM = re.compile(r"^[a-zA-Z0-9]+$")


def _send_validate_to(to: typing.Union[str, typing.List[str]]) -> None:
    """Calidate the to parameter for send."""
    if not isinstance(to, str) and not isinstance(to, list):
        raise exceptions.SmsError(
            f'the value of "to" is not valid, expecting a string or a list of string, '
            f'received "{to}"'
        )
    if isinstance(to, str):
        result = phone_number.check(value=to)
        if not result.valid:
            raise exceptions.SmsError(
                f'the value of "to" is not valid, {result.reason}'
            )
    if isinstance(to, list):
        first_invalid_result = next(
            filter(lambda result: not result.valid, map(phone_number.check, to)),
            None,
        )
        if first_invalid_result is not None:
            raise exceptions.SmsError(
                f'the value of "to" is not valid, {first_invalid_result.reason}'
            )


def _send_validate_from(from_: typing.Optional[str]) -> None:
    """Calidate the from_ parameter for send."""
    if from_ is not None:
        if not isinstance(from_, str):
            raise exceptions.SmsError(
                'the value of "from_" is not valid, expected a string, received '
                f'"{from_}"'
            )
        if len(from_) > 11:
            raise exceptions.SmsError(
                'the value of "from_" has too many characters, expected at most 11 '
                f'characters, received "{from_}"'
            )
        if not _VALID_FROM.search(from_):
            raise exceptions.SmsError(
                'the value of "from_" contains invalid characters, expected alpha '
                f'numeric characters, received "{from_}"'
            )
        result = phone_number.check(value=from_)
        if result.valid:
            raise exceptions.SmsError(
                'the value of "from_" is a phone number, expected alpha '
                f'numeric characters that are not phone numbers, received "{from_}"'
            )


def send(
    to: typing.Union[str, typing.List[str]],
    body: str,
    from_: typing.Optional[str] = None,
    validity: typing.Optional[int] = None,
    scheduled_delivery: typing.Optional[int] = None,
) -> TSms:
    """
    Send an SMS.

    Args:
        to: The destination mobile number.
        body: The body of the message.
        from: Alpha numeric sender identity.
        validity: How long the platform should attempt to deliver the message for
            (in minutes).
        scheduled_delivery: How long the platform should wait before attempting to send
            the message (in minutes).

    Returns:
        The message that was sent.

    """
    # Validate to
    _send_validate_to(to)
    # Validate body
    if not isinstance(body, str):
        raise exceptions.SmsError(
            f'the value of "body" is not valid, expected a string, received "{body}"'
        )
    # Validate from_
    _send_validate_from(from_)
    # Validate validity
    if validity is not None:
        if not isinstance(validity, int) or isinstance(validity, bool):
            raise exceptions.SmsError(
                'the value of "validity" is not valid, expected an integer, received '
                f'"{validity}"'
            )
    # Validate scheduled_delivery
    if scheduled_delivery is not None:
        if not isinstance(scheduled_delivery, int) or isinstance(
            scheduled_delivery, bool
        ):
            raise exceptions.SmsError(
                'the value of "scheduled_delivery" is not valid, expected an integer, '
                f'received "{scheduled_delivery}"'
            )

    try:
        token = oauth.get_token()
    except exceptions.CredentialError as exc:
        raise exceptions.SmsError(f"Could not retrieve an OAuth token: {exc}") from exc

    url = "https://tapi.telstra.com/v2/messages/sms"
    data: typing.Dict[str, typing.Union[str, typing.List[str], int]] = {
        "to": to,
        "body": body,
    }
    if from_ is not None:
        data["from"] = from_
    if validity is not None:
        data["validity"] = validity
    if scheduled_delivery is not None:
        data["scheduledDelivery"] = scheduled_delivery
    data_str = json.dumps(data).encode()
    headers = {
        "Content-Type": "application/json",
        "Authorization": token.authorization,
    }
    sms_request = request.Request(url, data=data_str, headers=headers, method="POST")
    try:
        with request.urlopen(sms_request) as response:
            sms_dict = json.loads(response.read().decode())
            return TSms(
                to=sms_dict["messages"][0]["to"],
                delivery_status=sms_dict["messages"][0]["deliveryStatus"],
                message_id=sms_dict["messages"][0]["messageId"],
                message_status_url=sms_dict["messages"][0]["messageStatusURL"],
            )
    except error.HTTPError as exc:
        raise exceptions.SmsError(f"Could not send SMS: {exc}") from exc



"""Helper for phone numbers."""

import dataclasses
import re
import typing


@dataclasses.dataclass
class Result:
    """
    Result of the check.

    Attrs:
        valid: Whether the phone number is valid.
        reason: If the phone number is not valid, the reason why.

    """

    valid: bool
    reason: typing.Optional[str]


_PHONE_REGEX = re.compile(r"^((\+614)|(04))\d{8}$")


def check(value: str) -> Result:
    """
    Check whether a phone number is valid.

    Args:
        value: The phone number to check.

    Returns:
        Whether the phone number is valid.

    """
    if not isinstance(value, str):
        return Result(valid=False, reason=f'expecting a string, received "{value}"')

    if not value.startswith("+614") and not value.startswith("04"):
        return Result(
            valid=False,
            reason=(
                "expecting a phone number starting with +614 or 04 followed by 8 "
                f'digits, received "{value}"'
            ),
        )

    if value.startswith("+614") and len(value) != 12:
        return Result(
            valid=False,
            reason=(
                "the phone number has an incorrect number of characters, expecting "
                f'12, received "{value}" which has {len(value)} characters'
            ),
        )

    if value.startswith("04") and len(value) != 10:
        return Result(
            valid=False,
            reason=(
                "the phone number has an incorrect number of characters, expecting "
                f'10, received "{value}" which has {len(value)} characters'
            ),
        )

    if _PHONE_REGEX.search(value) is None:
        return Result(
            valid=False,
            reason=(
                "the phone number contains invalid characters, expecting  a phone "
                "number starting with +614 or 04 followed by 8 digits, received "
                f'"{value}"'
            ),
        )

    return Result(valid=True, reason=None)



"""Handle subscriptions."""

import dataclasses
import json
from urllib import error, request

from . import exceptions, oauth

_URL = "https://tapi.telstra.com/v2/messages/provisioning/subscriptions"


@dataclasses.dataclass
class TSubscription:
    """
    A phone number subscription.

    Attrs:
        destination_address: The phone number that a message can be sent to.
        active_days: The number of days left on the subscription.

    """

    destination_address: str
    active_days: int


def create(active_days: int = 30) -> TSubscription:
    """
    Create a subscription.

    Raises SubscriptionError if anything goes wrong.

    Args:
        active_days: The number of days the subscription will be active.

    """
    try:
        token = oauth.get_token()
    except exceptions.CredentialError as exc:
        raise exceptions.SubscriptionError(
            f"Could not create subscription: {exc}"
        ) from exc

    data = json.dumps({"activeDays": active_days}).encode()
    headers = {
        "Content-Type": "application/json",
        "Authorization": token.authorization,
        "Cache-Control": "no-cache",
    }
    subscription_request = request.Request(
        _URL, data=data, headers=headers, method="POST"
    )
    try:
        with request.urlopen(subscription_request) as response:
            subscription_dict = json.loads(response.read().decode())
            return TSubscription(
                destination_address=subscription_dict["destinationAddress"],
                active_days=subscription_dict["activeDays"],
            )
    except error.HTTPError as exc:
        raise exceptions.SubscriptionError(
            f"Could not create subscription: {exc}"
        ) from exc


def get() -> TSubscription:
    """
    Retrieve current subscription.

    Raises SubscriptionError if anything goes wrong.

    """
    try:
        token = oauth.get_token()
    except exceptions.CredentialError as exc:
        raise exceptions.SubscriptionError(
            f"Could not retrieve subscription: {exc}"
        ) from exc

    headers = {
        "Content-Type": "application/json",
        "Authorization": token.authorization,
        "Cache-Control": "no-cache",
    }
    subscription_request = request.Request(_URL, headers=headers, method="GET")
    try:
        with request.urlopen(subscription_request) as response:
            subscription_dict = json.loads(response.read().decode())
            return TSubscription(
                destination_address=subscription_dict["destinationAddress"],
                active_days=subscription_dict["activeDays"],
            )
    except error.HTTPError as exc:
        raise exceptions.SubscriptionError(
            f"Could not delete subscription: {exc}"
        ) from exc


def delete() -> None:
    """
    Delete current subscription.

    Raises SubscriptionError if anything goes wrong.

    """
    try:
        token = oauth.get_token()
    except exceptions.CredentialError as exc:
        raise exceptions.SubscriptionError(
            f"Could not delete subscription: {exc}"
        ) from exc

    data = json.dumps({"emptyArr": 0}).encode()
    headers = {
        "Content-Type": "application/json",
        "Authorization": token.authorization,
        "Cache-Control": "no-cache",
    }
    subscription_request = request.Request(
        _URL, data=data, headers=headers, method="DELETE"
    )
    try:
        with request.urlopen(subscription_request):
            return
    except error.HTTPError as exc:
        raise exceptions.SubscriptionError(
            f"Could not delete subscription: {exc}"
        ) from exc



"""Handles OAuth interactions."""

import dataclasses
import json
import math
import time
from urllib import error, parse, request

from . import exceptions
from .utils import config


@dataclasses.dataclass
class TToken:
    """
    OAuth token.

    Attrs:
        access_token: The OAuth access token.
        token_type: The type of the token.
        expires_in: The number of seconds until the token expires.
        retrieved_at: The time when the token was retrieved.

        expired: Whether the token is expired.
        authorization: The value of the Authorization header with the token.

    """

    def __init__(self, access_token: str, token_type: str, expires_in: str):
        """Construct."""
        self.retrieved_at = math.ceil(time.time())
        self.access_token = access_token
        self.token_type = token_type
        self.expires_in = int(expires_in)

    # The access token
    access_token: str
    # The type of the token
    token_type: str
    # The time to expiry
    expires_in: int
    # The time it was created
    retrieved_at: int

    @property
    def expired(self):
        """Whether the tokens are expired."""
        return math.ceil(time.time()) >= self.retrieved_at + self.expires_in

    @property
    def authorization(self):
        """Create the value of the Authorization header."""
        return f"Bearer {self.access_token}"


_CACHE = {"old_token": None}


def _reuse_token(func):
    """Decorate to reuse tokens that are not expired."""

    def inner() -> TToken:
        """Decorator."""
        if _CACHE["old_token"] is None or _CACHE["old_token"].expired:
            _CACHE["old_token"] = func()
        assert _CACHE["old_token"] is not None
        return _CACHE["old_token"]

    return inner


def _get_token() -> TToken:
    """
    Retrieve the OAuth tokens.

    Returns:
        The oauth tokens.

    """
    url = "https://tapi.telstra.com/v2/oauth/token"
    data = parse.urlencode(
        {
            "grant_type": "client_credentials",
            "client_id": config.get().tls_client_key,
            "client_secret": config.get().tls_client_secret,
            "scope": "NSMS",
        }
    ).encode("ascii")
    try:
        with request.urlopen(url, data) as response:
            return TToken(**json.loads(response.read().decode()))
    except error.HTTPError as exc:
        raise exceptions.CredentialError(f"Could not retrieve token: {exc}") from exc


get_token = _reuse_token(_get_token)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tls', 'tls.messaging', 'tls.messaging.utils']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'tls.messaging',
    'version': '0.1.3',
    'description': 'SDK for the Telstra Messaging API',
    'long_description': '# Telstra Messaging\n\nThe SDK for the Telstra messaging API.\n\n## Installing\n\n```bash\npip install messaging\n```\n\n## Getting Started\n\nSet the `TLS_CLIENT_KEY` and `TLS_CLIENT_SECRET` environment variables. These\nare the `Client key` and `Client secret` you can find here:\n<https://dev.telstra.com/user/me/apps>.\n\nTo send your first SMS:\n\n```python\nfrom tls.messaging import sms\n\nsms.send(to="+61412345678", body="Hi")\n```\n\n```bash\nTLS_CLIENT_KEY="XXXX" TLS_CLIENT_SECRET="YYYY" python app.py\n```\n',
    'author': 'David Andersson',
    'author_email': 'david-andersson@users.noreply.github.com ',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



"""Used to work with BNUMs for the free trial."""

import json
import typing
from urllib import error, request

from . import exceptions, oauth
from .utils import phone_number

TPhoneNumbers = typing.List[str]
_URL = "https://tapi.telstra.com/v2/messages/freetrial/bnum"


def register(phone_numbers: TPhoneNumbers) -> TPhoneNumbers:
    """
    Register phone numbers as b party's in the free trial.

    Args:
        phone_numbers: The phone numbers to register.

    Returns:
        The phone numbers that have been registered.

    """
    if not isinstance(phone_numbers, list):
        raise exceptions.BnumError(
            f'invalid value for "phone_numbers" argument, expecting list of strings, '
            f'received "{phone_numbers}"'
        )
    result = next(
        filter(lambda result: not result.valid, map(phone_number.check, phone_numbers)),
        None,
    )
    if result is not None:
        raise exceptions.BnumError(
            f'invalid value for "phone_numbers" argument, {result.reason}'
        )

    try:
        token = oauth.get_token()
    except exceptions.CredentialError as exc:
        raise exceptions.BnumError(f"Could register phone numbers: {exc}") from exc

    data = json.dumps({"bnum": phone_numbers}).encode()
    headers = {
        "Content-Type": "application/json",
        "Authorization": token.authorization,
    }
    sms_request = request.Request(_URL, data=data, headers=headers, method="POST")
    try:
        with request.urlopen(sms_request) as response:
            bnum_dict = json.loads(response.read().decode())
            return bnum_dict["bnum"]
    except error.HTTPError as exc:
        raise exceptions.BnumError(f"Could register phone numbers: {exc}") from exc


def get() -> TPhoneNumbers:
    """
    Retrieve the registered phone numbers as b party's in the free trial.

    Returns:
        The phone numbers that have been registered.

    """
    try:
        token = oauth.get_token()
    except exceptions.CredentialError as exc:
        raise exceptions.BnumError(f"Could retrieve phone numbers: {exc}") from exc

    headers = {
        "Content-Type": "application/json",
        "Authorization": token.authorization,
    }
    sms_request = request.Request(_URL, headers=headers, method="GET")
    try:
        with request.urlopen(sms_request) as response:
            data = response.read().decode()
            bnum_dict = json.loads(data)
            return bnum_dict["bnum"]
    except error.HTTPError as exc:
        raise exceptions.BnumError(f"Could retrieve phone numbers: {exc}") from exc
